package uulm.teamname.marvelous.server.lobby.pipelining;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import uulm.teamname.marvelous.gamelibrary.messages.ParticipantType;
import uulm.teamname.marvelous.gamelibrary.requests.Request;
import uulm.teamname.marvelous.gamelibrary.requests.RequestBuilder;
import uulm.teamname.marvelous.gamelibrary.requests.RequestType;
import uulm.teamname.marvelous.server.lobbymanager.Participant;
import uulm.teamname.marvelous.server.netconnector.Client;

import static org.junit.jupiter.api.Assertions.*;
import static org.assertj.core.api.Assertions.*;

class PacketTest {
    Packet packet;

    @BeforeEach
    void beforeEach(){
        var requests = new Request[] {
                new RequestBuilder(RequestType.Req).buildGameRequest(),
                new RequestBuilder(RequestType.MoveRequest).buildGameRequest()
        };
        packet = new Packet(requests, null);
    }

    @Test
    void containsRequestTest(){
        assertThat(packet.containsRequestOfType(RequestType.Req)).isTrue();
        assertThat(packet.containsRequestOfType(RequestType.DisconnectRequest)).isFalse();
    }

    @Test
    void removeRequestsOfTypesTest(){
        packet.removeRequestsOfTypes(RequestType.Req);
        assertThat(packet).containsOnly(new RequestBuilder(RequestType.MoveRequest).buildGameRequest());
    }

    @Test
    void removeRequestsNotOfTypesTest(){
        packet.removeRequestsNotOfTypes(RequestType.Req);
        assertThat(packet).containsOnly(new RequestBuilder(RequestType.Req).buildGameRequest());
    }

    @Test
    void getOriginTest(){
        var requests = new Request[] {
                new RequestBuilder(RequestType.Req).buildGameRequest(),
                new RequestBuilder(RequestType.MoveRequest).buildGameRequest()
        };
        var participant = new Participant(new Client(null), "SomeLobby", ParticipantType.PlayerOne, false);
        packet = new Packet(requests, participant);

        assertThat(packet.getOrigin()).isEqualTo(participant);
    }

}
